import { defineComponent, ref, onMounted, onUnmounted } from 'vue';
import styles from './pptList.module.less';
import { vaildPPTUrl } from '@/helpers/utils';
import { storage } from '@/helpers/storage';
import { ACCESS_TOKEN } from '@/store/mutation-types';

export default defineComponent({
  name: 'pptList',
  props: {
    pptData: {
      type: Object,
      default: () => ({})
    },
    fromType: {
      type: String // 'PLATFORM' | 'TEACHER' | 'CLASS'
    }
  },
  emits: ['initPPT', 'changeSlideIndex', 'init'],
  setup(props, { emit, expose }) {
    const Authorization = storage.get(ACCESS_TOKEN);
    const iframeRef = ref<HTMLIFrameElement>();
    const src = `${vaildPPTUrl()}/#/mobileScreen?id=${
      props.pptData.id
    }&Authorization=${Authorization}&hideFullScreen=true&fromType=${
      props.fromType
    }`;
    // 上一页下一页
    function handleChangeSlide(type: 'prev' | 'next') {
      iframeRef.value?.contentWindow?.postMessage(
        { type: 'changePageSlide', content: type },
        '*'
      );
    }
    function handleMessage(event: any) {
      const { type, content } = event.data || {};
      console.log(content, 'message');
      if (type === 'initPPT') {
        emit('initPPT', content);
      } else if (type === 'changeSlideIndex') {
        emit('changeSlideIndex', content);
      }
    }
    emit('init');
    onMounted(() => {
      window.addEventListener('message', handleMessage);
    });
    onUnmounted(() => {
      window.removeEventListener('message', handleMessage);
    });
    expose({
      handleChangeSlide
    });
    return () => (
      <div class={styles.pptList}>
        <iframe
          ref={iframeRef}
          class={[styles.container]}
          frameborder="0"
          src={src}></iframe>
      </div>
    );
  }
});
